/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/util/log_entry_id.hh"

using namespace Util;

Optional<unsigned long long> Util::get_numeric_log_entry_id(const std::string& log_entry_id)
{
    // 94017576.20250516.domainbrowser.0
    // or
    // 94017576
    static constexpr char delimiter = '.';
    static constexpr int decimal_base = 10;
    std::size_t len = 0;
    const auto at_least_one_character_processed = [&len]() { return 0 < len; };
    const auto all_characters_processed = [&log_entry_id, &len]() { return log_entry_id.length() == len; };
    const auto stopped_at_the_delimiter = [&log_entry_id, &len]() { return log_entry_id[len] == delimiter; };
    try
    {
        const auto id = std::stoull(log_entry_id, &len, decimal_base);
        if (at_least_one_character_processed() &&
            (all_characters_processed() || stopped_at_the_delimiter()))
        {
            return id;
        }
    }
    catch (...) { }
    return {};
}
