/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "util/log_entry_id.hh"

#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>

#include <sstream>

BOOST_AUTO_TEST_SUITE(Tests)
BOOST_AUTO_TEST_SUITE(Util)
BOOST_AUTO_TEST_SUITE(LogEntryId)

namespace {

std::string create_log_entry_id(unsigned long long id)
{
    std::ostringstream out;
    out << id;
    return out.str();
}

std::string create_log_entry_id(unsigned long long id, const std::string& time_begin, const std::string& service, bool is_monitoring)
{
    std::ostringstream out;
    out << id << '.' << time_begin << '.' << service << '.' << (is_monitoring ? '1' : '0');
    return out.str();
}

}//namespace {anonymous}

BOOST_AUTO_TEST_CASE(correct_log_entry_id)
{
    {
        static constexpr auto id = 0ull;
        {
            const auto numeric_id = ::Util::get_numeric_log_entry_id(create_log_entry_id(id));
            BOOST_REQUIRE(numeric_id.is_set());
            BOOST_CHECK_EQUAL(numeric_id.get_value(), id);
        }
        {
            const auto numeric_id = ::Util::get_numeric_log_entry_id(create_log_entry_id(id, "20250516", "EPP", false));
            BOOST_REQUIRE(numeric_id.is_set());
            BOOST_CHECK_EQUAL(numeric_id.get_value(), id);
        }
    }
    {
        static constexpr auto id = 12345ull;
        {
            const auto numeric_id = ::Util::get_numeric_log_entry_id(create_log_entry_id(id));
            BOOST_REQUIRE(numeric_id.is_set());
            BOOST_CHECK_EQUAL(numeric_id.get_value(), id);
        }
        {
            const auto numeric_id = ::Util::get_numeric_log_entry_id(create_log_entry_id(id, "20250516", "WebWhois", true));
            BOOST_REQUIRE(numeric_id.is_set());
            BOOST_CHECK_EQUAL(numeric_id.get_value(), id);
        }
    }
}

BOOST_AUTO_TEST_CASE(incorrect_log_entry_id)
{
    BOOST_CHECK(!::Util::get_numeric_log_entry_id("").is_set());
    BOOST_CHECK(!::Util::get_numeric_log_entry_id("38446744073709551615").is_set());
    BOOST_CHECK(!::Util::get_numeric_log_entry_id("7,3").is_set());
    BOOST_CHECK(!::Util::get_numeric_log_entry_id("invalid").is_set());
    BOOST_CHECK(!::Util::get_numeric_log_entry_id(".invalid").is_set());
    BOOST_CHECK(!::Util::get_numeric_log_entry_id("invalid.").is_set());
    BOOST_CHECK(!::Util::get_numeric_log_entry_id("invalid.5").is_set());
}

BOOST_AUTO_TEST_SUITE_END()//Tests/Util/LogEntryId
BOOST_AUTO_TEST_SUITE_END()//Tests/Util
BOOST_AUTO_TEST_SUITE_END()//Tests
