/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef CONTACT_AUTHINFO_HH_A38C700E1607C084ACBB49F4A0AF5638//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define CONTACT_AUTHINFO_HH_A38C700E1607C084ACBB49F4A0AF5638

#include "src/common_types.hh"
#include "src/exceptions.hh"
#include "src/contact/contact_common_types.hh"
#include "src/registrar/registrar_common_types.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <chrono>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Contact {

struct AddContactAuthInfoRequest : Util::Printable<AddContactAuthInfoRequest>
{
    std::string contact_id;
    ContactId get_contact_id() const;
    boost::optional<SnapshotId> contact_snapshot_id;
    boost::optional<std::string> auth_info;
    boost::optional<std::chrono::seconds> ttl;
    Registrar::RegistrarHandle by_registrar;
    boost::optional<LogEntryId> log_entry_id;
    std::string to_string() const;
};

struct AddContactAuthInfoReply
{
    struct Data : Util::Printable<Data>
    {
        explicit Data();
        explicit Data(std::string auth_info);
        boost::optional<std::string> generated_auth_info;
        std::string to_string() const;
    };
    struct Exception
    {
        struct ContactDoesNotExist : Registry::Exception
        {
            explicit ContactDoesNotExist(const AddContactAuthInfoRequest&);
        };
        struct RegistrarDoesNotExist : Registry::Exception
        {
            explicit RegistrarDoesNotExist(const AddContactAuthInfoRequest&);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const AddContactAuthInfoRequest&, std::set<std::string> fields);
            std::set<std::string> fields;
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const AddContactAuthInfoRequest&);
        };
    };
};

AddContactAuthInfoReply::Data add_contact_auth_info(const AddContactAuthInfoRequest& request);

}//namespace Fred::Registry::Contact
}//namespace Fred::Registry
}//namespace Fred

#endif//CONTACT_AUTHINFO_HH_A38C700E1607C084ACBB49F4A0AF5638
