/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UPDATE_CONTACT_STATE_STREAM_HH_50DD86D9C4E6B6C52CC5386702B21AB3//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define UPDATE_CONTACT_STATE_STREAM_HH_50DD86D9C4E6B6C52CC5386702B21AB3

#include "src/common_types.hh"
#include "src/exceptions.hh"
#include "src/contact/contact_common_types.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <map>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Contact {

struct UpdateContactStateStreamRequest : Util::Printable<UpdateContactStateStreamRequest>
{
    using SetFlags = std::map<std::string, bool>;
    struct ContactIdLess
    {
        bool operator()(const ContactId&, const ContactId&) const noexcept;
    };
    std::map<ContactId, SetFlags, ContactIdLess> updates;
    boost::optional<SnapshotId> contact_snapshot_id;
    boost::optional<LogEntryId> log_entry_id;

    std::string to_string()const;
};

struct UpdateContactStateStreamReply
{
    struct Exception
    {
        struct ContactDoesNotExist : Registry::Exception
        {
            explicit ContactDoesNotExist(const UpdateContactStateStreamRequest&);
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const UpdateContactStateStreamRequest&);
        };
        struct StateFlagDoesNotExist : Registry::Exception
        {
            explicit StateFlagDoesNotExist(const UpdateContactStateStreamRequest&);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const UpdateContactStateStreamRequest&, std::set<std::string> fields);
            std::set<std::string> fields;
        };
    };
};

void update_contact_state_stream(const UpdateContactStateStreamRequest& request);

}//namespace Fred::Registry::Contact
}//namespace Fred::Registry
}//namespace Fred

#endif//UPDATE_CONTACT_STATE_STREAM_HH_50DD86D9C4E6B6C52CC5386702B21AB3
