/*
 * Copyright (C) 2019-2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DOMAIN_STATE_HH_9B528419466E403290036CE3364F1910//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define DOMAIN_STATE_HH_9B528419466E403290036CE3364F1910

#include "src/domain/domain_common_types.hh"
#include "src/exceptions.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <map>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Domain {

struct DomainStateRequest : Util::Printable<DomainStateRequest>
{
    std::string domain_id;
    DomainId get_domain_id() const;
    boost::optional<SnapshotId> domain_snapshot_id;
    std::string to_string() const;
};

struct DomainState : Util::Printable<DomainState>
{
    std::map<std::string, bool> flag_presents;
    std::string to_string() const;
};

struct DomainStateReply
{
    struct Data : Util::Printable<Data>
    {
        DomainId domain_id;
        DomainState state;
        SnapshotId snapshot_id;
        std::string to_string() const;
    };
    struct Exception
    {
        struct DomainDoesNotExist : Registry::Exception
        {
            explicit DomainDoesNotExist(const DomainStateRequest& request);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const DomainStateRequest&, std::set<std::string> fields);
            std::set<std::string> fields;
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const DomainStateRequest&);
        };
    };
};

DomainStateReply::Data domain_state(const DomainStateRequest& request);

} // namespace Fred::Registry::Domain
} // namespace Fred::Registry
} // namespace Fred

#endif//DOMAIN_STATE_HH_9B528419466E403290036CE3364F1910
