/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef KEYSET_AUTHINFO_HH_207766786078398C73E44B702D87FAD1//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define KEYSET_AUTHINFO_HH_207766786078398C73E44B702D87FAD1

#include "src/common_types.hh"
#include "src/exceptions.hh"
#include "src/keyset/keyset_common_types.hh"
#include "src/registrar/registrar_common_types.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <chrono>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Keyset {

struct AddKeysetAuthInfoRequest : Util::Printable<AddKeysetAuthInfoRequest>
{
    std::string keyset_id;
    KeysetId get_keyset_id() const;
    boost::optional<SnapshotId> keyset_snapshot_id;
    boost::optional<std::string> auth_info;
    boost::optional<std::chrono::seconds> ttl;
    Registrar::RegistrarHandle by_registrar;
    boost::optional<LogEntryId> log_entry_id;
    std::string to_string() const;
};

struct AddKeysetAuthInfoReply
{
    struct Data : Util::Printable<Data>
    {
        explicit Data();
        explicit Data(std::string auth_info);
        boost::optional<std::string> generated_auth_info;
        std::string to_string() const;
    };
    struct Exception
    {
        struct KeysetDoesNotExist : Registry::Exception
        {
            explicit KeysetDoesNotExist(const AddKeysetAuthInfoRequest&);
        };
        struct RegistrarDoesNotExist : Registry::Exception
        {
            explicit RegistrarDoesNotExist(const AddKeysetAuthInfoRequest&);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const AddKeysetAuthInfoRequest&, std::set<std::string> fields);
            std::set<std::string> fields;
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const AddKeysetAuthInfoRequest&);
        };
    };
};

AddKeysetAuthInfoReply::Data add_keyset_auth_info(const AddKeysetAuthInfoRequest& request);

}//namespace Fred::Registry::Keyset
}//namespace Fred::Registry
}//namespace Fred

#endif//KEYSET_AUTHINFO_HH_207766786078398C73E44B702D87FAD1
