/*
 * Copyright (C) 2019-2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

 #include "src/keyset/keyset_info_request.hh"
 #include "src/keyset/keyset_info_reply.hh"

#include "src/util/struct_to_string.hh"

#include <boost/uuid/string_generator.hpp>

namespace Fred {
namespace Registry {
namespace Keyset {

std::string KeysetInfoRequest::to_string() const
{
    return Util::StructToString().add("keyset_id", keyset_id)
                                 .add("keyset_history_id", keyset_history_id)
                                 .add("keyset_snapshot_id", keyset_snapshot_id)
                                 .add("include_domains_count", include_domains_count)
                                 .finish();
}

KeysetId KeysetInfoRequest::get_keyset_id() const
{
    try
    {
        return Util::make_strong<KeysetId>(boost::uuids::string_generator{}(keyset_id));
    }
    catch (...)
    {
        throw KeysetInfoReply::Exception::InvalidData{*this, {"keyset_id"}};
    }
}

KeysetHistoryId KeysetInfoRequest::get_keyset_history_id() const
{
    try
    {
        return Util::make_strong<KeysetHistoryId>(boost::uuids::string_generator{}(*keyset_history_id));
    }
    catch (...)
    {
        throw KeysetInfoReply::Exception::InvalidData{*this, {"keyset_history_id"}};
    }
}

std::string BatchKeysetInfoRequest::Request::to_string() const
{
    return Util::StructToString().add("keyset_id", keyset_id)
                                 .add("keyset_history_id", keyset_history_id)
                                 .add("keyset_snapshot_id", keyset_snapshot_id)
                                 .add("include_domains_count", include_domains_count)
                                 .finish();
}

std::string BatchKeysetInfoRequest::to_string() const
{
    return Util::StructToString().add("requests", requests)
                                 .finish();
}

std::string KeysetIdRequest::to_string() const
{
    return Util::StructToString().add("keyset_handle", keyset_handle)
                                 .finish();
}

} // namespace Fred::Registry::Keyset
} // namespace Fred::Registry
} // namespace Fred
