/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UPDATE_KEYSET_STATE_HH_2C5D0B69A5A63B81CD3FA4C1C90A6AEB//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define UPDATE_KEYSET_STATE_HH_2C5D0B69A5A63B81CD3FA4C1C90A6AEB

#include "src/common_types.hh"
#include "src/exceptions.hh"
#include "src/keyset/keyset_common_types.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <map>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Keyset {

struct UpdateKeysetStateRequest : Util::Printable<UpdateKeysetStateRequest>
{
    using SetFlags = std::map<std::string, bool>;
    struct KeysetIdLess
    {
        bool operator()(const KeysetId&, const KeysetId&) const noexcept;
    };
    std::map<KeysetId, SetFlags, KeysetIdLess> updates;
    boost::optional<SnapshotId> keyset_snapshot_id;
    boost::optional<LogEntryId> log_entry_id;

    std::string to_string()const;
};

struct UpdateKeysetStateReply
{
    struct Exception
    {
        struct KeysetDoesNotExist : Registry::Exception
        {
            explicit KeysetDoesNotExist(const UpdateKeysetStateRequest&);
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const UpdateKeysetStateRequest&);
        };
        struct StateFlagDoesNotExist : Registry::Exception
        {
            explicit StateFlagDoesNotExist(const UpdateKeysetStateRequest&);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const UpdateKeysetStateRequest&, std::set<std::string> fields);
            std::set<std::string> fields;
        };
    };
};

void update_keyset_state(const UpdateKeysetStateRequest& request);

}//namespace Fred::Registry::Keyset
}//namespace Fred::Registry
}//namespace Fred

#endif//UPDATE_KEYSET_STATE_HH_2C5D0B69A5A63B81CD3FA4C1C90A6AEB
