/*
 * Copyright (C) 2024-2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/nsset/list_nssets_by_contact_request.hh"
#include "src/util/struct_to_string.hh"

#include <stdexcept>

namespace Fred {
namespace Registry {
namespace Nsset {

ListNssetsByContactRequest::OrderBy::OrderBy(OrderByField field, OrderByDirection direction) noexcept
    : field{field},
      direction{direction}
{ }

std::string ListNssetsByContactRequest::OrderBy::to_string() const
{
    return Util::StructToString().add("field", Nsset::to_string(field))
                                 .add("direction", Registry::to_string(direction))
                                 .finish();
}

std::string ListNssetsByContactRequest::to_string() const
{
    return Util::StructToString().add("contact_id", contact_id)
                                 .add("contact_snapshot_id", contact_snapshot_id)
                                 .add("pagination", pagination)
                                 .add("order_by", order_by)
                                 .add("aggregate_entire_history", aggregate_entire_history)
                                 .finish();
}

} // namespace Fred::Registry::Nsset
} // namespace Fred::Registry
} // namespace Fred

using namespace Fred::Registry::Nsset;

std::string Fred::Registry::Nsset::to_string(ListNssetsByContactRequest::OrderByField field)
{
    switch (field)
    {
        case ListNssetsByContactRequest::OrderByField::unspecified:
            return "unspecified";
        case ListNssetsByContactRequest::OrderByField::crdate:
            return "crdate";
        case ListNssetsByContactRequest::OrderByField::handle:
            return "handle";
        case ListNssetsByContactRequest::OrderByField::domains_count:
            return "domains_count";
        case ListNssetsByContactRequest::OrderByField::sponsoring_registrar_handle:
            return "sponsoring_registrar_handle";
    }
    throw std::runtime_error{"unknown ListNssetsByContactRequest::OrderByField item"};
}
