/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef NSSET_AUTHINFO_HH_604A59F8A79E0BFEC373C6757FC5FFA2//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define NSSET_AUTHINFO_HH_604A59F8A79E0BFEC373C6757FC5FFA2

#include "src/common_types.hh"
#include "src/exceptions.hh"
#include "src/nsset/nsset_common_types.hh"
#include "src/registrar/registrar_common_types.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <chrono>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Nsset {

struct AddNssetAuthInfoRequest : Util::Printable<AddNssetAuthInfoRequest>
{
    std::string nsset_id;
    NssetId get_nsset_id() const;
    boost::optional<SnapshotId> nsset_snapshot_id;
    boost::optional<std::string> auth_info;
    boost::optional<std::chrono::seconds> ttl;
    Registrar::RegistrarHandle by_registrar;
    boost::optional<LogEntryId> log_entry_id;
    std::string to_string() const;
};

struct AddNssetAuthInfoReply
{
    struct Data : Util::Printable<Data>
    {
        explicit Data();
        explicit Data(std::string auth_info);
        boost::optional<std::string> generated_auth_info;
        std::string to_string() const;
    };
    struct Exception
    {
        struct NssetDoesNotExist : Registry::Exception
        {
            explicit NssetDoesNotExist(const AddNssetAuthInfoRequest& src);
        };
        struct RegistrarDoesNotExist : Registry::Exception
        {
            explicit RegistrarDoesNotExist(const AddNssetAuthInfoRequest&);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const AddNssetAuthInfoRequest& src, std::set<std::string> fields);
            std::set<std::string> fields;
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const AddNssetAuthInfoRequest&);
        };
    };
};

AddNssetAuthInfoReply::Data add_nsset_auth_info(const AddNssetAuthInfoRequest& request);

}//namespace Fred::Registry::Nsset
}//namespace Fred::Registry
}//namespace Fred

#endif//NSSET_AUTHINFO_HH_604A59F8A79E0BFEC373C6757FC5FFA2
