/*
 * Copyright (C) 2025  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UPDATE_NSSET_STATE_HH_3C4ED899C731760A207C862F44232BF7//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define UPDATE_NSSET_STATE_HH_3C4ED899C731760A207C862F44232BF7

#include "src/common_types.hh"
#include "src/exceptions.hh"
#include "src/nsset/nsset_common_types.hh"
#include "src/util/printable.hh"

#include <boost/optional.hpp>

#include <map>
#include <set>
#include <string>

namespace Fred {
namespace Registry {
namespace Nsset {

struct UpdateNssetStateRequest : Util::Printable<UpdateNssetStateRequest>
{
    using SetFlags = std::map<std::string, bool>;
    struct NssetIdLess
    {
        bool operator()(const NssetId&, const NssetId&) const noexcept;
    };
    std::map<NssetId, SetFlags, NssetIdLess> updates;
    boost::optional<SnapshotId> nsset_snapshot_id;
    boost::optional<LogEntryId> log_entry_id;

    std::string to_string()const;
};

struct UpdateNssetStateReply
{
    struct Exception
    {
        struct NssetDoesNotExist : Registry::Exception
        {
            explicit NssetDoesNotExist(const UpdateNssetStateRequest&);
        };
        struct InvalidSnapshot : Registry::Exception
        {
            explicit InvalidSnapshot(const UpdateNssetStateRequest&);
        };
        struct StateFlagDoesNotExist : Registry::Exception
        {
            explicit StateFlagDoesNotExist(const UpdateNssetStateRequest&);
        };
        struct InvalidData : Registry::Exception
        {
            explicit InvalidData(const UpdateNssetStateRequest&, std::set<std::string> fields);
            std::set<std::string> fields;
        };
    };
};

void update_nsset_state(const UpdateNssetStateRequest& request);

}//namespace Fred::Registry::Nsset
}//namespace Fred::Registry
}//namespace Fred

#endif//UPDATE_NSSET_STATE_HH_3C4ED899C731760A207C862F44232BF7
